#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_SET_LED    0
#define RQ_GET_SWITCH1 1
#define RQ_GET_SWITCH2 2
#define RQ_GET_SWITCH3 3
#define RQ_GET_SWITCH4 4
#define RQ_GET_SWITCH5 5

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    static uint16_t light_val;

    if (rq->bRequest == RQ_SET_LED) // RQ_SET_LED DEFINE = 0
    {
        uint8_t led_val = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];

        if (led_val==1){
			if (led_no == 1)
            	PORTC |= (1<<PC0);

			else if (led_no == 2)
            	PORTC |= (1<<PC2);

			else if (led_no == 3)
            	PORTC |= (1<<PC1);

			else if (led_no == 4)
            	PORTC |= (1<<PC3);

			else if (led_no == 5)
            	PORTD |= (1<<PD0);

			else if (led_no == 6)
            	PORTD |= (1<<PD1);

			else if (led_no == 7)
            	PORTB |= (1<<PB4);

			else if (led_no == 8)
            	PORTB |= (1<<PB2);

			else if (led_no == 9)
            	PORTB |= (1<<PB5);

			else if (led_no == 10)
            	PORTB |= (1<<PB3);
			else if (led_no == 11)
		PORTD |= (1<<PD6);
		}

        else if(led_val==0){
			if (led_no == 1)
            	PORTC &= ~(1<<PC0);

			else if (led_no == 2)
            	PORTC &= ~(1<<PC2 );

			else if (led_no == 3)
            	PORTC &= ~(1<<PC1 );

			else if (led_no == 4)
            	PORTC &= ~(1<<PC3 );

			else if (led_no == 5)
            	PORTD &= ~(1<<PD0 );

			else if (led_no == 6)
            	PORTD &= ~(1<<PD1 );

			else if (led_no == 7)
            	PORTB &= ~(1<<PB4 );

			else if (led_no == 8)
            	PORTB &= ~(1<<PB2 );

			else if (led_no == 9)
            	PORTB &= ~(1<<PB5 );

			else if (led_no == 10)
            	PORTB &= ~(1<<PB3 );
			else if(led_no == 11)
		PORTD &= ~(1<<PD6);
		}

        return 0;
    }

    else if (rq->bRequest == RQ_GET_SWITCH1) // RQ_GET_SWITCH = 1 
    {
        if ((PINC & (1<<PC4)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }
	else if (rq->bRequest == RQ_GET_SWITCH2) // RQ_GET_SWITCH = 1 
    {
        if ((PINC & (1<<PC5)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }
	else if (rq->bRequest == RQ_GET_SWITCH3) // RQ_GET_SWITCH = 1 
    {
        if ((PIND & (1<<PD5)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }
	else if (rq->bRequest == RQ_GET_SWITCH4) // RQ_GET_SWITCH = 1 
    {
        if ((PINB & (1<<PB0)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }
	else if (rq->bRequest == RQ_GET_SWITCH5) // RQ_GET_SWITCH = 1 
    {
        if ((PINB & (1<<PB1)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }

    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRB  |= (1<<PB2)|(1<<PB3)|(1<<PB4)|(1<<PB5); // PC0..PC2 = Output, PC3..PC5 = Input
	DDRC  |= (1<<PC0)|(1<<PC1)|(1<<PC2)|(1<<PC3);
	DDRD  |= (1<<PD0)|(1<<PD1)|(1<<PD6);

	//DDRB &= ~(1<<PB0)&~(1<<PB1);
	//DDRC &= ~(1<<PC4)&~(1<<PC5);
	//DDRD &= ~(1<<PD5);

    PORTB |= (1<<PB0)|(1<<PB1);                 // Enable pull-up at PC3
	PORTC |= (1<<PC4)|(1<<PC5); 
	PORTD |= (1<<PD5); 

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
